/* test pma, illustrate fallback to conventional memory allocator
   Copyright (C) 2022  Terence Kelly
   Contact:  tpkelly @ { acm.org, cs.princeton.edu, eecs.umich.edu }

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pma.h"

#define FP(...) (void)fprintf(stderr, __VA_ARGS__)

int main(int argc, char *argv[]) {
  int use_pma, r;  char *buf;
  if (3 > argc) {
    FP("usage: %s string1 string2 ... backingfile|\"NULL\"\n", argv[0]);
    return 1;
  }
  use_pma = strcmp("NULL", argv[argc-1]);
  if (use_pma) FP("using pma persistent memory allocator\n");
  else         FP("falling back on conventional (ephemeral) allocator\n");
  if (0 != (r = pma_init(2, use_pma ? argv[argc-1] : NULL))) {
    fprintf(stderr, "pma_init->%d pma_errno=%d\n", r, pma_errno);
    return 1;
  }
  buf = (char *) (use_pma ? pma_get_root() : NULL);
  // Main loop concatenates command-line arguments into a dynamically
  // growing buffer, printing buffer contents after each argument is
  // processed.
  for (int i = 1; i < argc-1; i++) {
    size_t buflen = buf ? strlen(buf    ) : 0,
           arglen =       strlen(argv[i]);
    char *tmp = (char *)pma_realloc(buf, buflen + arglen + 1);
    if (NULL == tmp) {
      FP("pma_realloc failed:  %serrno = %d: %s\n",
         use_pma ? "pma_"                 : "",
         use_pma ? pma_errno              : errno,
         use_pma ? "Use the Source, Luke" : strerror(errno));
      return 2;
    }
    buf = tmp;
    strcpy(buf + buflen, argv[i]);
    (void)printf("buffer: \"%s\"\n", buf);
  }
  if (use_pma)  pma_set_root(buf);  // make available for next run
  else          pma_free(buf);  // squelch Valgrind gripes
  return 0;
}

